;-------------------------------------------------------------------------------
;	FileName:E93LC56.asm
;-------------------------------------------------------------------------------
;
;       Author: 	Ricky
;       Company:	Holtek
;       Case:		
;	Device:		HT48E50, HT48E70
;	Revision:	1.0 
;       Date:		03-03-2004
;	Modify Date:
;       Compiler:	HT-IDE3000 V6.0 SP2 
;	Clock:		4MHz Cystral 
;
;-------------------------------------------------------------------------------
;
;    	<Function assignemt:>
;	93LC56 Command Subroutine, use 4 RAM, 2 Stack.
;
;-------------------------------------------------------------------------------
;
;       <Pin assignemt:>
;
;-------------------------------------------------------------------------------
              	#include      	E93LC56.INC

;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
RAM		.section 'data'
;-------------------------------------------------------------------------------
WRITE0	LABEL BYTE
READ0	LABEL BYTE
ERASE0	LABEL BYTE
EEADDR		db	?
WRITE1	LABEL BYTE
WRAL0	LABEL BYTE
EEDATA		db	?
cnt		db	?
Buf		db	?

;===============================================================================
;		Sub Program Area
;===============================================================================
;-------------------------------------------------------------------------------
ROM		.section 'code'
;-------------------------------------------------------------------------------

banksel:
		mov A,01h
		mov BP,A 				; set to bank 1
		mov A,040h
		mov MP1,A 				; set MP1 to EECR address
		ret


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	READ
;Function :	Read Data Command
;Input :	EEADDR
;Output :	EEDATA, A
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_READ:
		call banksel;
READ:
		set	CS			;CS=1

		mov	A,cREAD			;OP Code
		mov	Buf,A
		mov	A,4
		mov	cnt,A
		call	Shift

		mov	A,EEADDR		;Address
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		mov	A,00h			;Data
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		mov	A,Buf
		mov	EEDATA,A

		clr	CS			;CS=0
		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	ERASE
;Function :	Erase Command
;Input :	EEADDR
;Output :	None
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_ERASE:
		call banksel;
ERASE:
		set	CS			;CS=1

		mov	A,cERASE
		mov	Buf,A
		mov	A,4
		mov	cnt,A
		call	Shift

		mov	A,EEADDR
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		clr	CS			;CS=0

		call	ChkBusy
		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	WRITE
;Function :	Write Data Command
;Input :	EEADDR, EEDATA
;Output :	None
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_WRITE:
		call banksel;
WRITE:
		set	CS			;CS=1

		mov	A,cWRITE		;OP Code
		mov	Buf,A
		mov	A,4
		mov	cnt,A
		call	Shift

		mov	A,EEADDR		;Address
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		mov	A,EEDATA		;Data
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		clr	CS			;CS=0

		call	ChkBusy
		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	EWEN
;Function :	Erase/Write Enable Command
;Input :	None
;Output :	None
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_EWEN:
		call banksel;
EWEN:
		set	CS			;CS=1

		mov	A,cEWEN
		mov	Buf,A
		mov	A,4
		mov	cnt,A
		call	Shift

		mov	A,10000000b
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		clr	CS			;CS=0
		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	EWDS
;Function :	Erase/Write Disable Command
;Input :	None
;Output :	None
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_EWDS:
		call banksel;
EWDS:
		set	CS			;CS=1

		mov	A,cEWDS
		mov	Buf,A
		mov	A,4
		mov	cnt,A
		call	Shift

		mov	A,00000000b
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		clr	CS			;CS=0
		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	ERAL
;Function :	Erase All
;Input :	None
;Output :	None
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_ERAL:
		call banksel;
ERAL:
		set	CS			;CS=1

		mov	A,cERAL
		mov	Buf,A
		mov	A,4
		mov	cnt,A
		call	Shift

		mov	A,00000000b
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		clr	CS			;CS=0

		call	ChkBusy
		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	WRAL
;Function :	Write All Command
;Input :	EEDATA
;Output :	None
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_WRAL:
		call banksel;
WRAL:
		set	CS			;CS=1

		mov	A,cWRAL			;OP Code
		mov	Buf,A
		mov	A,4
		mov	cnt,A
		call	Shift

		mov	A,10000000b
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		mov	A,EEDATA		;Data
		mov	Buf,A
		mov	A,8
		mov	cnt,A
		call	Shift

		clr	CS			;CS=0

		call	ChkBusy
		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	Shift
;Function :
;Input :	Buf, cnt
;Output :	Buf
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Shift:
		sz	Buf.7			;Output
		jmp	DI_Hi
DI_Low:
		clr	DI
		jmp	CK_Hi
DI_Hi:
		set	DI

CK_Hi:
		set	SK			;CK=1
		rl	Buf
		sz	DO			;Input
		jmp	DO_Hi
DO_Low:
		clr	Buf.0
		jmp	CK_Low
DO_Hi:
		set	Buf.0
CK_Low:
		clr	SK			;CK=0

		sdz	cnt
		jmp	Shift

		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine :	ChkBusy
;Function :
;Input :	None
;Output :	None
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ChkBusy:
		set	CS			;CS=1
ChkBusy_Wait:
		nop
		nop
		set	SK			;CK=1
		nop
		sz	DO			;DO ?= 1
		jmp	ChkBusy_END
		nop
		nop
		clr	SK			;CK=0
		jmp	ChkBusy_Wait

ChkBusy_END:
		clr	SK
		clr	CS

		ret
